==> kahakai-0.6.1.tar.bz2: kahakai-0.6.1/src/RubyScript.cc <==
/* $Id: RubyScript.cc,v 1.18 2003/11/20 22:34:28 pervago Exp $ */

/**
 * @file   RubyScript.cc
 * @date   05-June-2003 10:49:54
 *
 * @brief Put brief explanation here
 *
 * Put a long explanation here.
 *
 */

/*
 * TODO
 * things need to be done with swig before this can be used
 */

#include "RubyScript.hh"
#include "ScriptFactory.hh"

#include <dlfcn.h>
#include <unistd.h>

#include <string>
#include <iostream>

using namespace std;

extern "C" {
    extern void Init_kahakai();
}

/*
 * file static
 */

// c++ doesnt have inline list-construction syntax
static list<string> makeSuffixes() {
    list<string> suffixes;
    suffixes.push_back("rb");
    return suffixes;
}

static Script * constructor(const string &filename) {
    return new RubyScript(filename);
}

/*
 * static initialization
 */

int RubyScript::_count = 0;

int RubyScript::_staticInit = doStaticInit();
int RubyScript::doStaticInit() {

    bool success = true;

    // register our Constructor with ScriptFactory
    Constructor ctor = constructor;
    success = success && ScriptFactory::registerScript(makeSuffixes(), ctor);

    // check success
    if (success)
        cout << "Loaded RubyScript" << endl;
    else
        // kill everything
        throw string("Failed to load RubyScript");

    // return value arbitrary
    return 0;
}

/*
 * public
 */

RubyScript::RubyScript(const string &filename)
    : Script(makeSuffixes(), filename)
{
    // if we are the first script, initialize the interpreter
    if (_count++ == 0) {
        initialize();
    }
}

RubyScript::RubyScript(const RubyScript &that)
    : Script(that) {
    // if we are the first script, initialize the interpreter
    if (_count++ == 0) {
        initialize();
    }
}

RubyScript::~RubyScript() {

    // if we were the last script, shutdown the interpreter
    if (--_count == 0) {
        finalize();
    }
}

bool RubyScript::run() {
    bool rv = false;
    string fname = getFilename();
    int error = 0;

    // basic error checking: check the suffix
    if (checkSuffix(fname)) {

    //Load the script into the interpreter
    rb_load_file(const_cast<char *>(fname.c_str()));

    //run the interpreter
    error = ruby_exec();
    if(error) {
            VALUE lasterr = rb_gv_get("$!");
            VALUE message = rb_obj_as_string(lasterr);
            cout << "message = " << RSTRING(message)->ptr << endl;
    
            
            
            if (!NIL_P(ruby_errinfo)) {
                VALUE ary = rb_funcall(ruby_errinfo, rb_intern("backtrace"), 
                                       0);
                int c;
                for (c=0; c<RARRAY(ary)->len; c++) {
                    cout << "\tfrom " << 
                        RSTRING(RARRAY(ary)->ptr[c])->ptr << 
                        "\n";
                }
            }
            throw string("Error running ruby script.");
            
        }

        rv = true;
    }

    return rv;
}

/*
 * protected
 */

/*
 * private
 */

typedef void (*void_func_t)();

void RubyScript::initialize() {
    string added_dirs = "$: << \".";  //start out with the cwd
    list<string> script_dirs = scriptDirs(); //Get the script dirs

    ruby_init();
    ruby_init_loadpath();
    ruby_script("embedded");


    void* handle = dlopen(PKGLIBDIR "/libkahakai_swig_ruby.so", RTLD_LAZY);
    if (!handle) {
        printf("couldn't open libkahakai_swig_ruby.so: %s\n", dlerror());
        exit(1);
    } 

    void_func_t _init_kahakai;

    _init_kahakai = (void_func_t)dlsym(handle, "Init_kahakai");

    if (!_init_kahakai) {
        printf("couldn't get Init_kahakai function: %s\n", dlerror());
        exit(1);
    }

    _init_kahakai();

    //insert the script dirs into the module path ($: global variable in ruby)
    list<string>::reverse_iterator it = script_dirs.rbegin();
    for (; it != script_dirs.rend(); it++) {
        added_dirs += "\" << \"" + *it;
    }
    added_dirs += "\"";

    rb_eval_string(const_cast<char *>(added_dirs.c_str()));
}

void RubyScript::finalize() {
    // no interpreter shutdown to be done, do nothing
    ruby_finalize();
}

==> kahakai-0.6.2.tar.bz2: kahakai-0.6.2/src/RubyScript.cc <==
/* $Id: RubyScript.cc,v 1.21 2004/02/04 21:53:28 brailsmt Exp $ */

/**
 * @file   RubyScript.cc
 * @date   05-June-2003 10:49:54
 *
 * @brief Put brief explanation here
 *
 * Put a long explanation here.
 *
 */

/*
 * TODO
 * things need to be done with swig before this can be used
 */

#include "RubyScript.hh"
#include "ScriptFactory.hh"

#include <dlfcn.h>
#include <unistd.h>

#include <string>
#include <iostream>

using namespace std;

extern "C" {
    extern void Init_kahakai();
}

/*
 * file static
 */

// c++ doesnt have inline list-construction syntax
static list<string> makeSuffixes() {
    list<string> suffixes;
    suffixes.push_back("rb");
    return suffixes;
}

static Script * constructor(const string &filename) {
	printf("info:  Creating a new RubyScript object from %s\n", filename.c_str());
    return new RubyScript(filename);
}

/*
 * static initialization
 */

int RubyScript::_count = 0;

int RubyScript::_staticInit = doStaticInit();
int RubyScript::doStaticInit() {

    bool success = true;

    // register our Constructor with ScriptFactory
    Constructor ctor = constructor;
    success = success && ScriptFactory::registerScript(makeSuffixes(), ctor);

    // check success
    if (success)
        printf("info:  Loaded RubyScript");
    else {
        // kill everything
        throw string("Failed to load RubyScript");
		printf("error:  Failed to load RubyScript");
	}

    // return value arbitrary
    return 0;
}

/*
 * public
 */

RubyScript::RubyScript(const string &filename)
    : Script(makeSuffixes(), filename)
{
    // if we are the first script, initialize the interpreter
	printf("info:  _count == %i\n", _count);
    if (_count++ == 0) {
        initialize();
    }
}

RubyScript::RubyScript(const RubyScript &that)
    : Script(that) {
    // if we are the first script, initialize the interpreter
    if (_count++ == 0) {
        initialize();
    }
}

RubyScript::~RubyScript() {

    // if we were the last script, shutdown the interpreter
    if (--_count == 0) {
        finalize();
    }
}

bool RubyScript::run() {
    bool rv = false;
    string fname = getFilename();
    int error = 0;

    // basic error checking: check the suffix
    if (checkSuffix(fname)) {

    //Load the script into the interpreter
    rb_load_file(const_cast<char *>(fname.c_str()));

    //run the interpreter
    error = ruby_exec();
    if(error) {
            VALUE lasterr = rb_gv_get("$!");
            VALUE message = rb_obj_as_string(lasterr);
            printf("rb_error:  message = %s", RSTRING(message)->ptr);
    
            if (!NIL_P(ruby_errinfo)) {
                VALUE ary = rb_funcall(ruby_errinfo, rb_intern("backtrace"), 
                                       0);
                int c;
                for (c=0; c<RARRAY(ary)->len; c++) {
                    cout << "\tfrom " << 
                        RSTRING(RARRAY(ary)->ptr[c])->ptr << 
                        "\n";
                }
            }
			printf("error:  Error running ruby script");
            throw string("Error running ruby script.");
        }

        rv = true;
    }

    return rv;
}

/*
 * protected
 */

/*
 * private
 */

typedef void (*void_func_t)();

void RubyScript::initialize() {
    string added_dirs = "$: << \".";  //start out with the cwd
    list<string> script_dirs = scriptDirs(); //Get the script dirs

    ruby_init();
    ruby_init_loadpath();
    ruby_script("embedded"); //What is this?

    void* handle = dlopen(PKGLIBDIR "/libkahakai_swig_ruby.so", RTLD_LAZY);
    if (!handle) {
        printf("critical:  couldn't open libkahakai_swig_ruby.so -- %s\n", dlerror());
        exit(1);
    } 

    void_func_t _init_kahakai;

    _init_kahakai = (void_func_t)dlsym(handle, "Init_kahakai");

    if (!_init_kahakai) {
        printf("critical:  couldn't get Init_kahakai function: %s\n", dlerror());
        exit(1);
    }

    _init_kahakai();

    //insert the script dirs into the module path ($: global variable in ruby)
    list<string>::reverse_iterator it = script_dirs.rbegin();
    for (; it != script_dirs.rend(); it++) {
        added_dirs += "\" << \"" + *it;
    }
    added_dirs += "\"";

    rb_eval_string(const_cast<char *>(added_dirs.c_str()));
}

void RubyScript::finalize() {
    // no interpreter shutdown to be done, do nothing
    ruby_finalize();
}
